@@ -1,4 +1,6 @@
-use std::{borrow::Borrow, fmt::{Debug, Display}, hash::Hash, marker::PhantomData, ops::Deref, pin::Pin};
+#![feature(negative_impls)]
+
+use std::{borrow::Borrow, convert::TryInto, fmt::{Debug, Display}, hash::Hash, marker::PhantomData, ops::Deref, pin::Pin};
 
 /// The interner.
 ///
@@ -10,6 +12,8 @@ pub struct Interner<'a, T: 'a + Eq> {
     _ph: PhantomData<&'a T>
 }
 
+impl<'a, T> !Sync for Interner<'a, T> {}
+
 /// The capacity of the first InternedItemHolder
 const BEGIN_INTERNER_CAPACITY: usize = 32;
 /// By how much every next interner's capacity changes
@@ -200,6 +204,31 @@ impl<'a, T: Hash> Hash for Intern<'a, T> {
     }
 }
 
+// Implement PartialOrd and Ord if the item implements it
+impl<'a, T: PartialOrd> PartialOrd for Intern<'a, T> {
+    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
+        self.0.partial_cmp(&other.0)
+    }
+}
+
+impl<'a, T: Ord> Ord for Intern<'a, T> {
+    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
+        self.0.cmp(&other.0)
+    }
+}
+
+impl<'a, T> Intern<'a, T> {
+    /// Create a new [`Intern`] type from a raw pointer.
+    ///
+    /// # Safety
+    /// The caller must guarantee that the pointed value is in fact
+    /// owned by an [`Interner`] which is still in scope (i.e. not dropped)
+    /// and that the pointer will not be mutated and/or moved.
+    pub unsafe fn from_raw(ptr: *const T) -> Self {
+        Intern(Pin::new_unchecked(ptr.as_ref().unwrap()))
+    }
+}
+
 pub struct Iter<'a, 'intern, T: std::cmp::Eq> {
     interner: &'a Interner<'intern, T>,
     holder_id: usize,
